@@ -7,19 +7,17 @@ use std::fs::read_to_string;
 use std::path::{Path, PathBuf};
 use std::time::Instant;
 use structopt::StructOpt;
-use two_timer;
 
 use lazy_static::lazy_static;
 use regex::Regex;
 
 use crate::commands::roi::Frequency;
 use crate::commands::{accounts, balance, commodities, payees, prices, register, roi, statistics};
+use crate::error::{MissingFileError, TimeParseError};
 use crate::models::Ledger;
-use crate::Error;
 use chrono::NaiveDate;
-use colored::Colorize;
 
-const VERSION: &'static str = env!("CARGO_PKG_VERSION");
+const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 #[derive(Debug, StructOpt)]
 enum Command {
@@ -136,6 +134,10 @@ pub struct CommonOpts {
     #[structopt(long = "--exchange", short = "-X")]
     pub exchange: Option<String>,
 
+    /// Add a conversion to the selected currency in the output
+    #[structopt(long = "--convert", short = "-C")]
+    pub convert: Option<String>,
+
     /// TODO Date format
     #[structopt(long = "--date-format", default_value = "%y-%b-%d")]
     pub date_format: String,
@@ -197,7 +199,7 @@ const LEDGER_PATHS: &str = ".ledgerrc";
 /// Load parameters from an external configuration file
 ///
 /// It checks whether ```--args-only``` has been passed so that the configuration file is ignored
-fn init_paths(args: Vec<String>) -> Result<Vec<String>, ()> {
+fn init_paths(args: Vec<String>) -> Result<Vec<String>, Box<dyn std::error::Error>> {
     let mut possible_paths: Vec<String> = Vec::new();
     let mut ignore_init = false;
     for i in 0..args.len() {
@@ -207,8 +209,9 @@ fn init_paths(args: Vec<String>) -> Result<Vec<String>, ()> {
         } else if args[i] == INIT_FILE_FLAG {
             let file = Path::new(&args[i + 1]);
             if !file.exists() {
-                eprintln!("Config file '{}' does not exist", args[i + 1]);
-                return Err(());
+                return Err(Box::new(MissingFileError::ConfigFileDoesNotExistError(
+                    file.to_path_buf(),
+                )));
             }
             possible_paths.push(args[i + 1].clone());
             continue;
@@ -226,7 +229,7 @@ fn init_paths(args: Vec<String>) -> Result<Vec<String>, ()> {
 }
 
 /// Entry point for the command line app
-pub fn run_app(input_args: Vec<String>) -> Result<(), ()> {
+pub fn run_app(input_args: Vec<String>) -> Result<(), Box<dyn std::error::Error>> {
     let mut config_file = None;
     let possible_paths = init_paths(input_args.clone())?;
     for path in possible_paths.iter() {
@@ -245,14 +248,14 @@ pub fn run_app(input_args: Vec<String>) -> Result<(), ()> {
     match Opt::from_iter_safe(args.iter()) {
         Err(error) => match Repl::from_iter_safe(args.iter()) {
             Ok(opt) => {
-                if opt.options.query.len() > 0 {
+                if !opt.options.query.is_empty() {
                     error.exit()
                 } else {
                     println!("dinero-rs v{}", VERSION);
 
                     let start = Instant::now();
 
-                    let mut ledger = Ledger::try_from(&opt.options).unwrap();
+                    let mut ledger = Ledger::try_from(&opt.options)?;
                     let duration = start.elapsed();
                     println!(
                         "Loaded ledger from {:?} in {:?}",
@@ -322,8 +325,8 @@ pub fn run_app(input_args: Vec<String>) -> Result<(), ()> {
     }
 }
 
-fn parse_config_file(file: &Path, input_args: &Vec<String>) -> Vec<String> {
-    let mut args = input_args.clone();
+fn parse_config_file(file: &Path, input_args: &[String]) -> Vec<String> {
+    let mut args = input_args.to_owned();
 
     let mut aliases = HashMap::new();
     // TODO you shouldn't have to do this manually
@@ -337,8 +340,8 @@ fn parse_config_file(file: &Path, input_args: &Vec<String>) -> Vec<String> {
     let contents = read_to_string(file).unwrap();
     for line in contents.lines() {
         let option = line.trim_start();
-        match option.chars().nth(0) {
-            Some(c) => match c {
+        if let Some(c) = option.chars().next() {
+            match c {
                 '-' => {
                     let message = format!(
                         "Bad config file {:?}. Only long option names allowed.\n{}",
@@ -353,10 +356,10 @@ fn parse_config_file(file: &Path, input_args: &Vec<String>) -> Vec<String> {
                         args.push(option.to_string());
                         let mut rest = String::new();
                         for arg in iter {
-                            rest.push_str(" ");
+                            rest.push(' ');
                             rest.push_str(arg);
                         }
-                        if rest.len() > 0 {
+                        if !rest.is_empty() {
                             args.push(rest.trim().to_string());
                         }
                     }
@@ -364,13 +367,15 @@ fn parse_config_file(file: &Path, input_args: &Vec<String>) -> Vec<String> {
                 ';' | '#' | '!' | '%' => (), // a comment
 
                 _ => panic!("Bad config file {:?}\n{}", file, line),
-            },
-            None => (),
+            }
         }
     }
     args
 }
-fn execute_command(opt: Opt, maybe_ledger: Option<Ledger>) -> Result<(), ()> {
+fn execute_command(
+    opt: Opt,
+    maybe_ledger: Option<Ledger>,
+) -> Result<(), Box<dyn std::error::Error>> {
     // Print options
     if let Err(e) = match opt.cmd {
         Command::Balance {
@@ -440,17 +445,14 @@ fn execute_command(opt: Opt, maybe_ledger: Option<Ledger>) -> Result<(), ()> {
             statistics::execute(&options, maybe_ledger)
         }
     } {
-        let err_str = format!("{}", e);
-        if err_str.len() > 0 {
-            eprintln!("{}", err_str);
-        }
-        return Err(());
+        Err(e)
+    } else {
+        Ok(())
     }
-    Ok(())
 }
 
 /// A parser for date expressions
-pub fn date_parser(date: &str) -> Result<NaiveDate, Error> {
+pub fn date_parser(date: &str) -> Result<NaiveDate, Box<dyn std::error::Error>> {
     lazy_static! {
         static ref RE_MONTH: Regex = Regex::new(r"(\d{4})[/-](\d\d?)$").unwrap();
         static ref RE_DATE: Regex = Regex::new(r"(\d{4})[/-](\d\d?)[/-](\d\d?)$").unwrap();
@@ -474,12 +476,7 @@ pub fn date_parser(date: &str) -> Result<NaiveDate, Error> {
             Ok((t1, _t2, _b)) => Ok(t1.date()),
             Err(e) => {
                 eprintln!("{:?}", e);
-                Err(Error {
-                    message: vec![format!("Invalid date {}", date)
-                        .as_str()
-                        .bold()
-                        .bright_red()],
-                })
+                Err(Box::new(TimeParseError {}))
             }
         }
     }
@@ -575,12 +572,12 @@ mod tests {
         let _res = run_app(args);
     }
     #[test]
-    #[should_panic]
     fn file_does_not_exist() {
         let args: Vec<String> = vec!["testing", "bal", "-f", "this_file_does_not_exist.ledger"]
             .iter()
             .map(|x| x.to_string())
             .collect();
-        let _res = run_app(args);
+        let res = run_app(args);
+        assert!(res.is_err())
     }
 }
