@@ -8,7 +8,6 @@ use crate::app::PeriodGroup;
 use crate::commands::balance::convert_balance;
 use crate::models::{conversion, Balance, Ledger, Money};
 use crate::parser::value_expr::build_root_node_from_expression;
-use crate::Error;
 use crate::{filter, CommonOpts};
 use chrono::{Datelike, Duration, NaiveDate};
 use num::{BigInt, BigRational, Zero};
@@ -24,18 +23,18 @@ pub fn execute(
     frequency: Frequency,
     calendar: bool,
     summary: bool,
-) -> Result<(), Error> {
-    let mut ledger = match maybe_ledger {
+) -> Result<(), Box<dyn std::error::Error>> {
+    let ledger = match maybe_ledger {
         Some(ledger) => ledger,
         None => Ledger::try_from(options)?,
     };
 
     // TODO exit gracefully
     assert!(
-        cash_flows_query.len() > 0,
+        !cash_flows_query.is_empty(),
         "cash flows query has to be provided"
     );
-    assert!(assets_value_query.len() > 0, "assets value query");
+    assert!(!assets_value_query.is_empty(), "assets value query");
     if calendar {
         assert!(frequency != Frequency::Yearly)
     }
@@ -63,7 +62,9 @@ pub fn execute(
 
     // Get a currency
     let mut currency = None;
-
+    if let Some(c) = &options.exchange {
+        currency = Some(ledger.commodities.get(c).unwrap().clone());
+    }
     let mut first_date = None;
     let mut last_date = None;
 
@@ -72,16 +73,16 @@ pub fn execute(
     for t in ledger.transactions.iter() {
         // cash_flows
         for p in t.postings.borrow().iter() {
-            if !filter::filter(&options, &cash_flows_node, t, p, &mut ledger.commodities)? {
+            if !filter::filter(options, &cash_flows_node, t, p, &ledger.commodities)? {
                 continue;
             }
             let index = get_period_index(p.date, &mut periods, frequency);
             let period = &mut periods[index];
 
             if first_date.is_none() {
-                first_date = Some(p.date.clone());
+                first_date = Some(p.date);
             }
-            last_date = Some(p.date.clone());
+            last_date = Some(p.date);
 
             match currency.as_ref() {
                 Some(_) => (),
@@ -109,7 +110,7 @@ pub fn execute(
 
         // balances
         for p in t.postings.borrow().iter() {
-            if !filter::filter(&options, &assets_value_node, t, p, &mut ledger.commodities)? {
+            if !filter::filter(options, &assets_value_node, t, p, &ledger.commodities)? {
                 continue;
             }
             let index = get_period_index(p.date, &mut periods, frequency);
@@ -126,13 +127,13 @@ pub fn execute(
     let mut last_period_date = None;
     for p in periods.iter_mut() {
         if last_period_date.is_none() {
-            last_period_date = Some(p.end.clone());
+            last_period_date = Some(p.end);
             continue;
         }
         // Because the gap may be more than one month, we need a loop
         'inner: loop {
             let expected = last_period_date.unwrap() + Duration::days(1);
-            last_period_date = Some(period_ending(expected.clone(), frequency));
+            last_period_date = Some(period_ending(expected, frequency));
             if expected == p.start {
                 break 'inner;
             }
@@ -146,7 +147,6 @@ pub fn execute(
 
     let mut prev_final_balance = Balance::new();
     let mut prev_final_money = None;
-
     for (i, p) in periods.iter_mut().enumerate() {
         if i > 0 {
             p.initial_balance = prev_final_balance;
@@ -176,7 +176,7 @@ pub fn execute(
         prev_final_money = p.final_money.clone();
     }
     match calendar {
-        false => print_normal(&periods, &options),
+        false => print_normal(&periods, options),
         true => print_calendar(&periods, &frequency),
     }
 
@@ -194,6 +194,7 @@ pub fn execute(
         total_twr -= 1.0;
         let num_days = ((last_date.unwrap() - first_date.unwrap()).num_days() + 1) as f64;
         let twr_annualized = (1.0 + total_twr).powf(365.25 / num_days) - 1.0;
+        println!("Currency used for calculations: {}", currency.unwrap());
         println!("Total TWR: {:.2}%", total_twr * 100.0);
         println!("Period: {:.2} years", num_days / 365.25);
         println!("Annualized TWR: {:.2}%", twr_annualized * 100.0);
@@ -201,7 +202,7 @@ pub fn execute(
     Ok(())
 }
 
-fn print_calendar(periods: &Vec<Period>, frequency: &Frequency) {
+fn print_calendar(periods: &[Period], frequency: &Frequency) {
     let mut table = Table::new();
     table.set_format(*format::consts::FORMAT_NO_LINESEP_WITH_TITLE);
 
@@ -276,7 +277,7 @@ fn print_calendar(periods: &Vec<Period>, frequency: &Frequency) {
     table.printstd();
 }
 
-fn print_normal(periods: &Vec<Period>, options: &CommonOpts) {
+fn print_normal(periods: &[Period], options: &CommonOpts) {
     let mut table = Table::new();
     table.set_format(*format::consts::FORMAT_NO_LINESEP_WITH_TITLE);
     table.set_titles(
@@ -290,7 +291,7 @@ fn print_normal(periods: &Vec<Period>, options: &CommonOpts) {
             r->format!("{}", p.cash_flow),
             r->format!("{}", p.final_money.as_ref().unwrap()),
             r->format!("{:.2}%", (&p.twr() * BigInt::from(100)).to_f64().unwrap()),
-            r->format!("{:.2}%", &p.twr_annualized() * 100 as f64),
+            r->format!("{:.2}%", p.twr_annualized() * 100_f64),
         ]);
     }
     // Print the table to stdout
@@ -346,8 +347,7 @@ impl Period {
             };
         }
         // (end - initial + flow) / initial
-        let twr = (end - &initial + flow) / initial;
-        twr
+        (end - &initial + flow) / initial
     }
 
     fn twr_annualized(&self) -> f64 {
