@@ -6,9 +6,10 @@ use std::rc::{Rc, Weak};
 use chrono::NaiveDate;
 use num::rational::BigRational;
 
+use crate::error::LedgerError;
 use crate::models::balance::Balance;
 use crate::models::{Account, Comment, HasName, Money, Payee};
-use crate::{LedgerError, List};
+use crate::List;
 use num::BigInt;
 use std::fmt;
 use std::fmt::{Display, Formatter};
@@ -77,8 +78,8 @@ impl<T> Transaction<T> {
                         current.push(c)
                     }
                 }
-                parts.push(current.clone());
-                //self.description.split(" ").map(|x| x.to_string()).collect();
+                parts.push(current);
+                //self.description.split(' ').map(|x| x.to_string()).collect();
                 let res = preprocess_query(&parts, &false);
                 self.filter_query = Some(res.clone());
                 res
@@ -142,7 +143,7 @@ impl Posting {
             date,
             balance: None,
             cost: None,
-            kind: kind,
+            kind,
             comments: vec![],
             tags: RefCell::new(vec![]),
             payee: Some(Rc::new(payee.clone())),
@@ -204,7 +205,7 @@ impl<PostingType> Transaction<PostingType> {
     }
 }
 
-fn total_balance(postings: &Vec<Posting>, kind: PostingType) -> Balance {
+fn total_balance(postings: &[Posting], kind: PostingType) -> Balance {
     let bal = Balance::new();
     postings
         .iter()
@@ -257,8 +258,7 @@ impl Transaction<Posting> {
             .borrow()
             .iter()
             .filter(|p| p.amount.is_none() & p.balance.is_none())
-            .collect::<Vec<&Posting>>()
-            .len()
+            .count()
     }
 
     /// Balances the transaction
@@ -275,14 +275,14 @@ impl Transaction<Posting> {
         &mut self,
         balances: &mut HashMap<Rc<Account>, Balance>,
         skip_balance_check: bool,
-    ) -> Result<Balance, LedgerError> {
+    ) -> Result<Balance, Box<dyn std::error::Error>> {
         let mut transaction_balance = Balance::new();
 
         // 1. Check the virtual postings
         match total_balance(&*self.postings.borrow(), PostingType::VirtualMustBalance).can_be_zero()
         {
             true => {}
-            false => return Err(LedgerError::TransactionIsNotBalanced),
+            false => return Err(Box::new(LedgerError::TransactionIsNotBalanced)),
         }
 
         // 1. Iterate over postings
@@ -308,7 +308,7 @@ impl Transaction<Posting> {
                                 "Difference:  {}",
                                 expected_balance - Balance::from(balance.clone())
                             );
-                            return Err(LedgerError::TransactionIsNotBalanced);
+                            return Err(Box::new(LedgerError::TransactionIsNotBalanced));
                         }
                     }
                 }
@@ -354,7 +354,7 @@ impl Transaction<Posting> {
                 postings.push(Posting {
                     account: p.account.clone(),
                     amount: p.amount.clone(),
-                    date: p.date.clone(),
+                    date: p.date,
                     balance: p.balance.clone(),
                     cost: p.cost.clone(),
                     kind: PostingType::Real,
@@ -364,7 +364,7 @@ impl Transaction<Posting> {
                     transaction: p.transaction.clone(),
                     origin: PostingOrigin::FromTransaction,
                 });
-            } else if &p.balance.is_some() & !skip_balance_check {
+            } else if p.balance.is_some() & !skip_balance_check {
                 // There is a balance
                 let balance = p.balance.as_ref().unwrap();
 
@@ -377,7 +377,7 @@ impl Transaction<Posting> {
                 balances.insert(p.account.clone(), Balance::from(balance.clone()));
                 postings.push(Posting {
                     account: p.account.clone(),
-                    date: p.date.clone(),
+                    date: p.date,
                     amount: Some(money),
                     balance: p.balance.clone(),
                     cost: p.cost.clone(),
@@ -392,7 +392,7 @@ impl Transaction<Posting> {
                 // We do nothing, but this is the account for the empty post
                 fill_account = p.account.clone();
                 fill_payee = p.payee.clone();
-                fill_date = p.date.clone();
+                fill_date = p.date;
             }
         }
 
@@ -404,7 +404,7 @@ impl Transaction<Posting> {
             .count()
             - postings.len();
         if empties > 1 {
-            Err(LedgerError::TooManyEmptyPostings(empties))
+            Err(Box::new(LedgerError::TooManyEmptyPostings(empties)))
         } else if empties == 0 {
             match transaction_balance.can_be_zero() {
                 true => {
@@ -415,13 +415,13 @@ impl Transaction<Posting> {
                             .borrow_mut()
                             .iter()
                             .filter(|p| p.kind != PostingType::Real)
-                            .map(|p| p.clone())
+                            .cloned()
                             .collect(),
                     );
                     self.postings.replace(postings);
                     Ok(transaction_balance)
                 }
-                false => Err(LedgerError::TransactionIsNotBalanced),
+                false => Err(Box::new(LedgerError::TransactionIsNotBalanced)),
             }
         } else {
             // Fill the empty posting
@@ -441,7 +441,7 @@ impl Transaction<Posting> {
                     comments: self.comments.clone(),
                     tags: RefCell::new(self.tags.clone()),
                     payee: fill_payee.clone(),
-                    date: fill_date.clone(),
+                    date: fill_date,
                     transaction: self.postings.borrow()[0].transaction.clone(),
                     origin: PostingOrigin::FromTransaction,
                 });
@@ -453,7 +453,7 @@ impl Transaction<Posting> {
                     .get_mut()
                     .iter()
                     .filter(|p| p.kind != PostingType::Real)
-                    .map(|p| p.clone())
+                    .cloned()
                     .collect(),
             );
             self.postings.replace(postings);
